;; Generated package description from mode: lisp-data; .el  -*- el-jobno-byte-compile: t -*-
(define-package "el-job" "2.4.4" "Contrived way to call a function using all CPU cores" '((emacs "29.1")) :commit "eb3058e44df3d694cd7b24e0bbb6f92e831ea53d" :authors '(("Martin Edström" . "meedstrom@runbox.eu")) :maintainer '("Martin Edström" . "meedstrom@runbox.eu") :keywords '("processes") :url "https://github.com/meedstrom/el-job")
